/*
Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const express = require('express');
const cookieParser = require('cookie-parser');
const bodyParser = require('body-parser');
const methodOverride = require('method-override');
const cors = require('cors');
const squel = require('squel');

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

process.env.NODE_ENV = process.env.NODE_ENV || 'development';
const config = require(`${libs}/config`);

const app = express();

const api = require('./routes/api');


app.use(bodyParser.json({limit: '50mb'}));
app.use(bodyParser.urlencoded({ extended: true, limit: '50mb' }));
app.use(cookieParser());
// Enable Cross-Origin Resource Sharing (CORS)
app.use(cors());
app.use(methodOverride());
// Middleware tha adds the squel object to req
app.use((req, res, next) => {
    req.resetSql = () => {
        req.sql = squel.select();
        // HACK to know wich table we are using
        req.sql.oldFrom = req.sql.from;
        req.sql.from = (name, alias = null) => {
            req.sql.oldFrom(name, alias || null);
            req.sql.tableFrom = name;
            return req.sql;
        };
    };
    req.resetSql();

    req.sql.oldClone = req.sql.clone;
    req.sql.clone = () => {
        let cloned = req.sql.oldClone();
        cloned.oldFrom = cloned.from;
        cloned.from = (name, alias = null) => {
            cloned.oldFrom(name, alias || null);
            cloned.tableFrom = name;
            return cloned;
        };
        return cloned;
    };

    next();
});

app.use((req, res, next) => {
    res.setHeader('Last-Modified', (new Date()).toUTCString());
    next();
});
// Mounts all API routes under /api/v1
app.use('/api/v1', api);

// Catch 404 and forward to error handler
app.use((req, res, next) => {
    res.status(404);
    log.error('%s %d %s', req.method, res.statusCode, req.url);
    res.json({ error: 'Error 404: Page not found' }).end();
});

// Express' default error handler
app.use((err, req, res, next) => {
    res.status(err.status || 500);
    log.error('%s %d %s', req.method, res.statusCode, err.message);
    log.error(`Route: ${req.originalUrl}`);
    log.error(err);
    res.json({ error: err.message }).end();
});

module.exports = app;
