/*
Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const libs = `${process.cwd()}/libs`;
const config = require(`${libs}/config`);

const winston = require('winston');

winston.emitErrs = true;

function getFilePath(module) {
    // using filename in log statements
    return module.filename.split('/').slice(-2).join('/');
}

function logger(module) {
    const logPath = `${process.cwd()}/logs`;
    const maxLogFiles = 5;
    const maxLogSize = 5242880;
    const log = new winston.Logger({
        transports: [
            new winston.transports.File({
                name: 'info-log',
                level: 'info',
                filename: `${logPath}/simcaq-info.log`,
                handleException: true,
                json: false,
                maxSize: maxLogSize, // 5MB
                maxFiles: maxLogFiles,
                colorize: false,
            }),
            new winston.transports.File({
                name: 'error-log',
                level: 'error',
                filename: `${logPath}/simcaq-error.log`,
                handleException: true,
                json: false,
                maxSize: maxLogSize, // 5MB
                maxFiles: maxLogFiles,
                colorize: false,
            }),
            new winston.transports.Console({
                name: 'debug-log',
                level: (process.env.NODE_ENV === 'development') ? 'debug' : 'error',
                label: getFilePath(module),
                handleException: true,
                json: false,
                colorize: true,
            }),
        ],
        exitOnError: false,
    });
    if (!config.debug) {
        log.remove('debug-log');
    }
    return log;
}

module.exports = logger;
