/*
Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const curPath = process.cwd();
const libs = `${process.cwd()}/libs`;
const log = require(`${libs}/log`)(module);
const chalk = require('chalk');
const packageConf = require(`${curPath}/package.json`);

module.exports = () => {
    // Parse version number from strings such as 'v4.2.0' or `>=4.0.0'
    function parseVersionNumber(versionString) {
        return parseFloat(versionString.replace(/[^\d\.]/g, ''));
    }
    // Ensure minimum supported node version is used
    const minNodeVersion = parseVersionNumber(packageConf.engines.node);
    const currentNodeVersion = parseVersionNumber(process.version);
    if (minNodeVersion > currentNodeVersion) {
        log.error(chalk.red(`You must upgrade node to >=${minNodeVersion}.x to use simcaq-node!`));
        return false;
    } else {
        log.info('Node version should work!');
        return true;
    }
};
