/*
Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const libs = `${process.cwd()}/libs`;
const log = require(`${libs}/log`)(module);
const db = require(`${libs}/db/query_exec`);

 // Middleware that executes a query defined by a squel object in req.sql
function query(req, res, next) {
    let sql = req.sql.toParam();
    log.info(`Executando query ${req.sql.toString()}`);
    sql.text = sql.text.replace(/`/g, "");
    execute(sql.text, sql.values, (err, result) => {
        if(err) {
            log.error(err.stack);
            next(new Error('Request could not be satisfied due to a database error.'));
        } else {
            req.result = result;
            next();
        }
    });
}

function execute(text, values, cb) {
    db.execSqlQuery(text, values).then((result) => {
        cb(null, result);
    }, (error) => {
        log.error(error.stack);
        cb(new Error('Request could not be satisfied due to a database error.'));
    });
}

module.exports = {query, execute};
