/*
Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of SMPPIR-Node.

SMPPIR-Node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SMPPIR-Node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SMPPIR-Node.  If not, see <https://www.gnu.org/licenses/>.
*/



const express = require('express');

const admissionApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const rqf = new ReqQueryFields();
rqf.addField({
    name: 'filter',
    field: false,
    where: true,
}).addField({
    name: 'dims',
    field: true,
    where: false,
})
.addValue({
    name: 'shift',
    table: 'admission_ag',
    tableField: 'turno_aluno',
    resultField: 'ds_turno_aluno',
    where: {
        relation: '=',
        type: 'string',
        field: 'turno_aluno',
    },
})
.addValue({
    name: 'gender',
    table: 'admission_ag',
    tableField: 'descricao_genero_aluno',
    resultField: 'ds_sexo_aluno',
    where: {
        relation: '=',
        type: 'string',
        field: 'descricao_genero_aluno',
    },
})
.addValue({
    name: 'ethnic',
    table: 'admission_ag',
    tableField: 'cor_raca_aluno',
    resultField: 'ds_cor_raca_aluno',
    where: {
        relation: '=',
        type: 'string',
        field: 'cor_raca_aluno',
    },
})
.addValue({
    name: 'course',
    table: 'admission_ag',
    tableField: 'nome_curso',
    resultField: 'no_curso',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome_curso',
    },
})
.addValue({
    name: 'institution',
    table: 'admission_ag',
    tableField: 'nome_ies',
    resultField: 'no_ies',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome_ies',
    },
})
.addValue({
    name: 'institution_cod',
    table: 'admission_ag',
    tableField: 'cod_ies',
    resultField: 'co_ies',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cod_ies',
    },
})
.addValue({
    name: 'year',
    table: 'admission_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo',
    },
})
.addValue({
    name: 'min_year',
    table: 'admission_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo',
    },
})
.addValue({
    name: 'max_year',
    table: 'admission_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo',
    },
});

admissionApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.field('admission_ag.ano_censo as no_ano_pesquisa')
        .field('SUM(total) AS total')
        .from('admission_ag')
        .group('admission_ag.ano_censo')
        .order('admission_ag.ano_censo');
    next();
}, query, response('admission'));

module.exports = admissionApp;
