/*
Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of SMPPIR-Node.

SMPPIR-Node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SMPPIR-Node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SMPPIR-Node.  If not, see <https://www.gnu.org/licenses/>.
*/



const express = require('express');

const cad_unico = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true,
}).addField({
    name: 'dims',
    field: true,
    where: false,
})
.addValue({
    name: 'year',
    table: 'ciganos_rights_ag',
    tableField: 'ano_censo',
    resultField: 'ano_pesquisa',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo',
    },
})

.addValue({
    name: 'family_type',
    table: 'ciganos_rights_ag',
    tableField: 'ind_tipo_familia',
    resultField: 'ind_parc_mds_fam',
    where: {
        relation: '=',
        type: 'string',
        field: 'ind_tipo_familia',
    },
})

.addValue({
    name: 'education_families_conclude',
    table: 'ciganos_rights_ag',
    tableField: 'concluiu_frequentou',
    resultField: 'cod_concluiu_frequentou_memb',
    where: {
        relation: '=',
        type: 'string',
        field: 'concluiu_frequentou',
    },
})

.addValue({
    name: 'education_families_level',
    table: 'ciganos_rights_ag',
    tableField: 'ano_serie_frequentou',
    resultField: 'cod_ano_serie_frequentou_memb',
    where: {
        relation: '=',
        type: 'string',
        field: 'ano_serie_frequentou',
    },
})

.addValue({
    name: 'education_families',
    table: 'ciganos_rights_ag',
    tableField: 'curso_frequentou',
    resultField: 'cod_curso_frequentou_pessoa_memb',
    where: {
        relation: '=',
        type: 'string',
        field: 'curso_frequentou',
    },
})



;


cad_unico.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    console.log(req.sql.toString());
    req.sql.field('SUM(total) AS Total')
        .from('ciganos_rights_ag');
    next();
}, query, response('cad_unico'));

module.exports = cad_unico;
