/*
Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of SMPPIR-Node.

SMPPIR-Node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SMPPIR-Node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SMPPIR-Node.  If not, see <https://www.gnu.org/licenses/>.
*/



const express = require('express');

const cad_unico_count = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true,
}).addField({
    name: 'dims',
    field: true,
    where: false,
})
.addValue({
    name: 'year',
    table: 'quilombola_eixo_3_ag',
    tableField: 'ano_censo',
    resultField: 'ano_pesquisa',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo',
    },
})

.addValue({
    name: 'family_type',
    table: 'quilombola_eixo_3_ag',
    tableField: 'ind_tipo_familia',
    resultField: 'ind_parc_mds_fam',
    where: {
        relation: '=',
        type: 'string',
        field: 'ind_tipo_familia',
    },
})


.addValue({
    name:'cod_home_bolsa_familia',
    table:'quilombola_eixo_3_ag',
    tableField:'marca_programa_bolsa_familia',
    resultField:'marc_pbf',
    where: {
        relation:'=',
        type:'string',
        field:'marca_programa_bolsa_familia',
    },
})

.addValue({
    name:'quilombola_family',
    table:'quilombola_eixo_3_ag',
    tableField:'ind_familia_quilombola',
    resultField:'ind_familia_quilombola_fam',
    where: {
        relation:'=',
        type:'string',
        field:'ind_familia_quilombola',
    },
})

.addValue({
    name:'cod_agriculture_job',
    table:'quilombola_eixo_3_ag',
    tableField:'atividade_extrativista',
    resultField:'cod_agricultura_trab_memb',
    where: {
        relation:'=',
        type:'string',
        field:'atividade_extrativista',
    },
})


.addValue({
    name: 'per_capita_income',
    table: 'quilombola_eixo_3_ag',
    tableField: 'renda_media',
    resultField: 'vlr_renda_media_fam',
    where: {
        relation: '=',
        type: 'float',
        field: 'renda_media',
    },
})

.addValue({
    name: 'home_location',
    table: 'quilombola_eixo_3_ag',
    tableField: 'codigo_ibge',
    resultField: 'cd_ibge',
    where: {
        relation: '=',
        type: 'string',
        field: 'codigo_ibge',
    },
})

.addValue({
    name: 'uf',
    table: 'quilombola_eixo_3_ag',
    tableField: 'sigla_estado',
    resultField: 'uf',
    where: {
        relation: '=',
        type: 'string',
        field: 'sigla_estado',
    },
})
.addValue({
    name: 'region',
    table: 'quilombola_eixo_3_ag',
    tableField : 'nome_regiao',
    resultField: 'region',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome_regiao',
    },
})

;


cad_unico_count.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    console.log(req.sql.toString());
    req.sql.field('SUM(total) AS Total')
        .from('quilombola_eixo_3_ag');
    next();
}, query, response('cad_unico'));

module.exports = cad_unico_count;
