/*
Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of SMPPIR-Node.

SMPPIR-Node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SMPPIR-Node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SMPPIR-Node.  If not, see <https://www.gnu.org/licenses/>.
*/



const express = require('express');

const cad_unico = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true,
}).addField({
    name: 'dims',
    field: true,
    where: false,
})
.addValue({
    name: 'year',
    table: 'quilombola_eixo_4_ag',
    tableField: 'ano_censo',
    resultField: 'ano_pesquisa',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo',
    },
})

.addValue({
    name:'cod_local_home',
    table:'quilombola_eixo_4_ag',
    tableField:'cod_local_domicilio',
    resultField:'cod_local_domic_fam',
    where: {
        relation:'=',
        type:'string',
        field:'cod_local_domicilio',
    },
})

.addValue({
    name:'quilombola_family',
    table:'quilombola_eixo_4_ag',
    tableField:'ind_familia_quilombola',
    resultField:'ind_familia_quilombola_fam',
    where: {
        relation:'=',
        type:'string',
        field:'ind_familia_quilombola',
    },
})

.addValue({
    name:'age',
    table:'quilombola_eixo_4_ag',
    tableField:'idade',
    resultField:'idade',
    where: {
        relation:'=',
        type:'string',
        field:'idade',
    },
})


.addValue({
    name: 'education_families_conclude',
    table: 'quilombola_eixo_4_ag',
    tableField: 'concluiu_frequentou',
    resultField: 'cod_concluiu_frequentou_memb',
    where: {
        relation: '=',
        type: 'string',
        field: 'concluiu_frequentou',
    },
})
.addValue({
    name: 'education_families_level',
    table: 'quilombola_eixo_4_ag',
    tableField: 'ano_serie_frequentou',
    resultField: 'cod_ano_serie_frequentou_memb',
    where: {
        relation: '=',
        type: 'string',
        field: 'ano_serie_frequentou',
    },
})
.addValue({
    name: 'education_families',
    table: 'quilombola_eixo_4_ag',
    tableField: 'curso_frequentou',
    resultField: 'cod_curso_frequentou_pessoa_memb',
    where: {
        relation: '=',
        type: 'string',
        field: 'curso_frequentou',
    },
})
.addValue({
    name: 'uf',
    table: 'quilombola_eixo_4_ag',
    tableField: 'sigla_estado',
    resultField: 'uf',
    where: {
        relation: '=',
        type: 'string',
        field: 'sigla_estado',
    },
})

.addValue({
    name: 'region',
    table: 'quilombola_eixo_4_ag',
    tableField : 'nome_regiao',
    resultField: 'region',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome_regiao',
    },
})

.addValue({
    name:'gender',
    table:'quilombola_eixo_4_ag',
    tableField:'genero',
    resultField:'genero',
    where: {
        relation:'=',
        type:'string',
        field:'genero',
    },
})


;


cad_unico.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    console.log(req.sql.toString());
    req.sql.field('SUM(total) AS Total')
        .from('quilombola_eixo_4_ag');
    next();
}, query, response('cad_unico'));

module.exports = cad_unico;
