/*
Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of SMPPIR-Node.

SMPPIR-Node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SMPPIR-Node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SMPPIR-Node.  If not, see <https://www.gnu.org/licenses/>.
*/



const express = require('express');

const fiesApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true,
}).addField({
    name: 'dims',
    field: true,
    where: false,
})
.addValue({
    name: 'gender',
    table: 'fies_ag',
    tableField: 'genero',
    resultField: 'ds_sexo',
    where: {
        relation: '=',
        type: 'string',
        field: 'genero',
    },
})
.addValue({
    name: 'ethnic',
    table: 'fies_ag',
    tableField: 'raca_cor',
    resultField: 'ds_raca_cor',
    where: {
        relation: '=',
        type: 'string',
        field: 'raca_cor',
    },
})
.addValue({
    name: 'course',
    table: 'fies_ag',
    tableField: 'curso',
    resultField: 'ds_curso',
    where: {
        relation: '=',
        type: 'string',
        field: 'curso',
    },
})
.addValue({
    name: 'institution',
    table: 'fies_ag',
    tableField: 'nome_ies',
    resultField: 'no_ies',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome_ies',
    },
})
.addValue({
    name: 'institution_cod',
    table: 'fies_ag',
    tableField: 'codigo_ies',
    resultField: 'co_ies',
    where: {
        relation: '=',
        type: 'integer',
        field: 'codigo_ies',
    },
})
.addValue({
    name: 'offer_location',
    table: 'fies_ag',
    tableField: 'campus',
    resultField: 'no_campus',
    where: {
        relation: '=',
        type: 'string',
        field: 'campus',
    },
})
.addValue({
    name: 'offer_location_cod',
    table: 'fies_ag',
    tableField: 'codigo_campus',
    resultField: 'co_campus',
    where: {
        relation: '=',
        type: 'integer',
        field: 'codigo_campus',
    },
})
.addValue({
    name: 'loan',
    table: 'fies_ag',
    tableField: 'percentual_solicitado_financiamento',
    resultField: 'nu_percent_solicitado_financ',
    where: {
        relation: '=',
        type: 'string',
        field: 'percentual_solicitado_financiamento',
    },
})
.addValue({
    name: 'loan_type',
    table: 'fies_ag',
    tableField: 'tipo_fianca',
    resultField: 'tp_fianca',
    where: {
        relation: '=',
        type: 'string',
        field: 'tipo_fianca',
    },
})
.addValue({
    name: 'uf',
    table: 'fies_ag',
    tableField: 'sigla_uf',
    resultField: 'sg_uf',
    where: {
        relation: '=',
        type: 'string',
        field: 'sigla_uf',
    },
})
.addValue({
    name: 'region',
    table: 'fies_ag',
    tableField: 'regiao_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'regiao_id',
    },
})
.addValue({
    name: 'year',
    table: 'fies_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo',
    },
})
.addValue({
    name: 'min_year',
    table: 'fies_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo',
    },
})
.addValue({
    name: 'max_year',
    table: 'fies_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo',
    },
});

fiesApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.field('ano_censo as nu_ano')
        .field('COUNT(*) AS total')
        .from('fies_ag')
        .group('ano_censo')
        .order('ano_censo');
    next();
}, query, response('fies'));

module.exports = fiesApp;
