/*
Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of SMPPIR-Node.

SMPPIR-Node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SMPPIR-Node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SMPPIR-Node.  If not, see <https://www.gnu.org/licenses/>.
*/



const express = require('express');

const ibge_census = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

let rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'ethnic',
    table: 'pessoa_censo',
    tableField: 'co_cor_raca',
    resultField: 'co_cor_raca',
    where: {
        relation: '=',
        type: 'integer',
        field: 'co_cor_raca'
    }
}).addValue({
    name: 'gender',
    table: 'pessoa_censo',
    tableField: 'co_sexo',
    resultField: 'co_sexo',
    where: {
        relation: '=',
        type: 'integer',
        field: 'co_sexo'
    }
}).addValue({
    name: 'religion',
    table: 'pessoa_censo',
    tableField: 'co_religiao',
    resultField: 'co_religiao',
    where: {
        relation: '=',
        type: 'integer',
        field: 'co_religiao'
    }
}).addValue({
    name: 'income_range',
    table: 'pessoa_censo',
    tableField: 'nu_sal_min_todos_trabalhos',
    resultField: 'nu_sal_min_todos_trabalhos',
    where: {
        relation: '=',
        type: 'double',
        field: 'nu_sal_min_todos_trabalhos'
    }
}).addValue({
    name: 'age',
    table: 'pessoa_censo',
    tableField: 'nu_idade_calculada_anos',
    resultField: 'nu_idade_calculada_anos',
    where: {
        relation: '=',
        type: 'integer',
        field: 'nu_idade_calculada_anos'
    }
}).addValue({
    name: 'age_greater_than',
    table: 'pessoa_censo',
    tableField: 'nu_idade_calculada_anos',
    resultField: 'nu_idade_calculada_anos',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'nu_idade_calculada_anos'
    }
}).addValue({
    name: 'age_lesser_than',
    table: 'pessoa_censo',
    tableField: 'nu_idade_calculada_anos',
    resultField: 'nu_idade_calculada_anos',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'nu_idade_calculada_anos'
    }
}).addValue({
    name: 'highest_education',
    table: 'pessoa_censo',
    tableField: 'co_curso_mais_elevado_freq',
    resultField: 'co_curso_mais_elevado_freq',
    where: {
        relation: '=',
        type: 'string',
        field: 'co_curso_mais_elevado_freq'
    }
}).addValue({
    name: 'school_daycare',
    table: 'pessoa_censo',
    tableField: 'co_freq_escola_creche',
    resultField: 'co_freq_escola_creche',
    where: {
        relation: '=',
        type: 'integer',
        field: 'co_freq_escola_creche'
    }
}).addValue({
    name: 'work_status',
    table: 'pessoa_censo',
    tableField: 'co_condicao_atividade_semana_ref',
    resultField: 'co_condicao_atividade_semana_ref',
    where: {
        relation: '=',
        type: 'integer',
        field: 'co_condicao_atividade_semana_ref'
    }
}).addValue({
    name: 'education_level',
    table: 'pessoa_censo',
    tableField: 'co_nivel_instrucao',
    resultField: 'co_nivel_instrucao',
    where: {
        relation: '=',
        type: 'integer',
        field: 'co_nivel_instrucao'
    }
}).addValue({
    name: 'ocupation_position_and_category',
    table: 'pessoa_censo',
    tableField: 'co_posicao_na_ocupacao_e_categoria_do_emprego',
    resultField: 'co_posicao_na_ocupacao_e_categoria_do_emprego',
    where: {
        relation: '=',
        type: 'integer',
        field: 'co_posicao_na_ocupacao_e_categoria_do_emprego'
    }
}).addValue({
    name: 'home_status',
    table: 'domicilio_censo',
    tableField: 'co_situacao_dom',
    resultField: 'co_situacao_dom',
    where: {
        relation: '=',
        type: 'integer',
        field: 'co_situacao_dom'
    },
    join: {
        primary: ['co_controle','co_uf'],
        foreign: ['co_controle','co_uf'],
        foreignTable: 'pessoa_censo'
    }
}).addValue({
    name: 'work_condition',
    table: 'pessoa_censo',
    tableField: 'co_condicao_ocupacao_semana_ref',
    resultField: 'co_condicao_ocupacao_semana_ref',
    where: {
        relation: '=',
        type: 'integer',
        field: 'co_condicao_ocupacao_semana_ref'
    }
}).addValue({
    name: 'mensal_home_income_range',
    table: 'domicilio_censo',
    tableField: 'vl_rendimneto_mensal_domicilio_sal_min_per_capita',
    resultField: 'vl_rendimneto_mensal_domicilio_sal_min_per_capita',
    where: {
        relation: '=',
        type: 'integer',
        field: 'vl_rendimneto_mensal_domicilio_sal_min_per_capita'
    },
    join: {
        primary: ['co_controle','co_uf'],
        foreign: ['co_controle','co_uf'],
        foreignTable: 'pessoa_censo'
    }
}).addValue({
    name: 'relation_homeowner',
    table: 'pessoa_censo',
    tableField: 'co_relacao_responsavel_dom',
    resultField: 'co_relacao_responsavel_dom',
    where: {
        relation: '=',
        type: 'integer',
        field: 'co_relacao_responsavel_dom'
    }
})

;


ibge_census.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.field("COUNT(*) AS Total")
        .from("pessoa_censo")
    next();
}, query, response('ibge_census'));

module.exports = ibge_census;
