/*
Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of SMPPIR-Node.

SMPPIR-Node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SMPPIR-Node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SMPPIR-Node.  If not, see <https://www.gnu.org/licenses/>.
*/



const express = require('express');

const pnadApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true,
}).addField({
    name: 'dims',
    field: true,
    where: false,
}).addValue({
    name: 'city_name',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'no_cidade',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome',
    },
}).addValue({
    name: 'city_code',
    table: 'municipio',
    tableField: 'id',
    resultField: 'co_cidade',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id',
    },
})
.addValue({
    name: 'state',
    table: 'municipio',
    tableField: 'estado_id',
    resultField: 'sgl_estado',
    where: {
        relation: '=',
        type: 'integer',
        field: 'estado_id',
    },
});

pnadApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.field('id as co_cidade')
        .field('nome as no_cidade')
        .field('estado_id as sgl_estado')
        .from('municipio');

    next();
}, query, response('pnad'));

module.exports = pnadApp;
