/*
Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of SMPPIR-Node.

SMPPIR-Node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SMPPIR-Node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SMPPIR-Node.  If not, see <https://www.gnu.org/licenses/>.
*/



const express = require('express');

const pnadApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true,
}).addField({
    name: 'dims',
    field: true,
    where: false,
})
.addValue({
    name: 'ethnic',
    table: 'pnad_ag',
    tableField: 'cor_raca',
    resultField: 'co_cor_raca',
    where: {
        relation: '=',
        type: 'string',
        field: 'cor_raca',
    },
})
.addValue({
    name: 'type_of_school',
    table: 'pnad_ag',
    tableField: 'in_escola_rede',
    resultField: 'cod_rede_ensino',
    where: {
        relation: '=',
        type: 'string',
        field: 'in_escola_rede',
    },
})
// .addValue({
//     name: 'type_of_public_school',
//     table: 'pnad_ag',
//     tableField: 'area_rede_publica_ensino',
//     resultField: 'co_area_rede_publica_ensino',
//     where: {
//         relation: '=',
//         type: 'string',
//         field: 'area_rede_publica_ensino',
//     },
// })
.addValue({
    name: 'level_of_education',
    table: 'pnad_ag',
    tableField: 'nivel_escola',
    resultField: 'co_curso',
    where: {
        relation: '=',
        type: 'string',
        field: 'nivel_escola',
    },
})
.addValue({
    name: 'uf',
    table: 'pnad_ag',
    tableField: 'sigla',
    resultField: 'sgl_uf',
    where: {
        relation: '=',
        type: 'string',
        field: 'sigla',
    },
})
.addValue({
    name: 'region',
    table: 'pnad_ag',
    tableField: 'regiao_id',
    resultField: 'no_regiao',
    where: {
        relation: '=',
        type: 'integer',
        field: 'regiao_id',
    },
})
.addValue({
    name: 'year',
    table: 'pnad_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo',
    },
})
.addValue({
    name: 'min_year',
    table: 'pnad_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo',
    },
})
.addValue({
    name: 'max_year',
    table: 'pnad_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo',
    },
});

pnadApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.field('ano_censo as nu_ano_referencia')
        .field('SUM(total) AS total')
        .from('pnad_ag')
        .group('ano_censo')
        .order('ano_censo');
    next();
}, query, response('pnad'));

module.exports = pnadApp;
