/*
Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of SMPPIR-Node.

SMPPIR-Node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SMPPIR-Node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SMPPIR-Node.  If not, see <https://www.gnu.org/licenses/>.
*/



const express = require('express');

const prouniApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true,
}).addField({
    name: 'dims',
    field: true,
    where: false,
})
.addValue({
    name: 'shift',
    table: 'prouni_ag',
    tableField: 'nome_turno_curso',
    resultField: 'nome_turno_curso_bolsa',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome_turno_curso',
    },
})
.addValue({
    name: 'gender',
    table: 'prouni_ag',
    tableField: 'genero_beneficiario',
    resultField: 'sexo_beneficiario_bolsa',
    where: {
        relation: '=',
        type: 'string',
        field: 'genero_beneficiario',
    },
})
.addValue({
    name: 'ethnic',
    table: 'prouni_ag',
    tableField: 'raca_beneficiario',
    resultField: 'raca_beneficiario_bolsa',
    where: {
        relation: '=',
        type: 'string',
        field: 'raca_beneficiario',
    },
})
.addValue({
    name: 'course',
    table: 'prouni_ag',
    tableField: 'nome_curso',
    resultField: 'nome_curso_bolsa',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome_curso',
    },
})
.addValue({
    name: 'institution',
    table: 'prouni_ag',
    tableField: 'nome_ies',
    resultField: 'nome_ies_bolsa',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome_ies',
    },
})
.addValue({
    name: 'scholarship',
    table: 'prouni_ag',
    tableField: 'tipo',
    resultField: 'tipo_bolsa',
    where: {
        relation: '=',
        type: 'string',
        field: 'tipo',
    },
})
.addValue({
    name: 'uf',
    table: 'prouni_ag',
    tableField: 'sigla_uf_beneficiario',
    resultField: 'sigla_uf_beneficiario_bolsa',
    where: {
        relation: '=',
        type: 'string',
        field: 'sigla_uf_beneficiario',
    },
})
.addValue({
    name: 'region',
    table: 'prouni_ag',
    tableField: 'regiao_beneficiario',
    resultField: 'regiao_beneficiario_bolsa',
    where: {
        relation: '=',
        type: 'string',
        field: 'regiao_beneficiario',
    },
})
.addValue({
    name: 'year',
    table: 'prouni_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo',
    },
})
.addValue({
    name: 'min_year',
    table: 'prouni_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo',
    },
})
.addValue({
    name: 'max_year',
    table: 'prouni_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo',
    },
});

prouniApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.field('ano_censo as ano_concessao_bolsa')
        .field('SUM(total) AS total')
        .from('prouni_ag')
        .group('ano_censo')
        .order('ano_censo');
    next();
}, query, response('prouni'));

module.exports = prouniApp;
