/*
Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of SMPPIR-Node.

SMPPIR-Node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SMPPIR-Node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SMPPIR-Node.  If not, see <https://www.gnu.org/licenses/>.
*/



const express = require('express');

const student_loansApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true,
}).addField({
    name: 'dims',
    field: true,
    where: false,
})
.addValue({
    name: 'shift',
    table: 'student_loans_ag',
    tableField: 'turno_aluno',
    resultField: 'ds_turno_aluno',
    where: {
        relation: '=',
        type: 'string',
        field: 'turno_aluno',
    },
})
.addValue({
    name: 'gender',
    table: 'student_loans_ag',
    tableField: 'descricao_genero_aluno',
    resultField: 'ds_sexo_aluno',
    where: {
        relation: '=',
        type: 'string',
        field: 'descricao_genero_aluno',
    },
})
.addValue({
    name: 'ethnic',
    table: 'student_loans_ag',
    tableField: 'cor_raca_aluno',
    resultField: 'ds_cor_raca_aluno',
    where: {
        relation: '=',
        type: 'string',
        field: 'cor_raca_aluno',
    },
})
.addValue({
    name: 'course',
    table: 'student_loans_ag',
    tableField: 'nome_curso',
    resultField: 'no_curso',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome_curso',
    },
})
.addValue({
    name: 'institution_cod',
    table: 'student_loans_ag',
    tableField: 'cod_ies',
    resultField: 'co_ies',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cod_ies',
    },
})
.addValue({
    name: 'institution',
    table: 'student_loans_ag',
    tableField: 'nome_ies',
    resultField: 'no_ies',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome_ies',
    },
})
.addValue({
    name: 'refund_fies',
    table: 'student_loans_ag',
    tableField: 'financiamento_reembolsavel_fies',
    resultField: 'in_fin_reemb_fies',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_reembolsavel_fies',
    },
})
.addValue({
    name: 'refund_state',
    table: 'student_loans_ag',
    tableField: 'financiamento_reembolsavel_estadual',
    resultField: 'in_fin_reemb_estadual',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_reembolsavel_estadual',
    },
})
.addValue({
    name: 'refund_city',
    table: 'student_loans_ag',
    tableField: 'financiamento_reembolsavel_municipal',
    resultField: 'in_fin_reemb_municipal',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_reembolsavel_municipal',
    },
})
.addValue({
    name: 'refund_edu_inst',
    table: 'student_loans_ag',
    tableField: 'financiamento_reembolsavel_prog_ies',
    resultField: 'in_fin_reemb_prog_ies',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_reembolsavel_prog_ies',
    },
})
.addValue({
    name: 'refund_org',
    table: 'student_loans_ag',
    tableField: 'financiamento_reembolsavel_entidade_externa',
    resultField: 'in_fin_reemb_ent_externa',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_reembolsavel_entidade_externa',
    },
})
.addValue({
    name: 'refund_others',
    table: 'student_loans_ag',
    tableField: 'financiamento_reembolsavel_outra',
    resultField: 'in_fin_reemb_outra',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_reembolsavel_outra',
    },
})
.addValue({
    name: 'no_refund_prouni_integral',
    table: 'student_loans_ag',
    tableField: 'financiamento_naoreembolsavel_prouni_integral',
    resultField: 'in_fin_naoreemb_prouni_integr',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_naoreembolsavel_prouni_integral',
    },
})
.addValue({
    name: 'no_refund_prouni_partial',
    table: 'student_loans_ag',
    tableField: 'financiamento_naoreembolsavel_prouni_parcial',
    resultField: 'in_fin_naoreemb_prouni_parcial',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_naoreembolsavel_prouni_parcial',
    },
})
.addValue({
    name: 'no_refund_state',
    table: 'student_loans_ag',
    tableField: 'financiamento_naoreembolsavel_estadual',
    resultField: 'in_fin_naoreemb_estadual',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_naoreembolsavel_estadual',
    },
})
.addValue({
    name: 'no_refund_city',
    table: 'student_loans_ag',
    tableField: 'financiamento_naoreembolsavel_municipal',
    resultField: 'in_fin_naoreemb_municipal',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_naoreembolsavel_municipal',
    },
})
.addValue({
    name: 'no_refund_edu_inst',
    table: 'student_loans_ag',
    tableField: 'financiamento_naoreembolsavel_prog_ies',
    resultField: 'in_fin_naoreemb_prog_ies',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_naoreembolsavel_prog_ies',
    },
})
.addValue({
    name: 'no_refund_org',
    table: 'student_loans_ag',
    tableField: 'financiamento_naoreembolsavel_entidade_externa',
    resultField: 'in_fin_naoreemb_ent_externa',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_naoreembolsavel_entidade_externa',
    },
})
.addValue({
    name: 'no_refund_others',
    table: 'student_loans_ag',
    tableField: 'financiamento_naoreembolsavel_outra',
    resultField: 'in_fin_naoreemb_outra',
    where: {
        relation: '=',
        type: 'string',
        field: 'financiamento_naoreembolsavel_outra',
    },
})
.addValue({
    name: 'year',
    table: 'student_loans_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo',
    },
})
.addValue({
    name: 'min_year',
    table: 'student_loans_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo',
    },
})
.addValue({
    name: 'max_year',
    table: 'student_loans_ag',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo',
    },
});

student_loansApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.field('student_loans_ag.ano_censo as no_ano_pesquisa')
        .field('SUM(total) AS total')
        .from('student_loans_ag')
        .group('student_loans_ag.ano_censo')
        .order('student_loans_ag.ano_censo');
    next();
}, query, response('student_loans'));

module.exports = student_loansApp;
