/*
Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of SMPPIR-Node.

SMPPIR-Node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

SMPPIR-Node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SMPPIR-Node.  If not, see <https://www.gnu.org/licenses/>.
*/



const express = require('express');

const vacancyApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

let rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'uf',
    table: 'vagas_ofertadas',
    tableField: 'uf',
    where: {
        relation: '=',
        type: 'string',
        field: 'uf'
    }
}).addValue({
    name: 'region',
    table: 'vagas_ofertadas',
    tableField: 'regiao',
    where: {
        relation: '=',
        type: 'string',
        field: 'regiao'
    }
}).addValue({
    name: 'cod_ies',
    table: 'vagas_ofertadas',
    tableField: 'cod_ies',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cod_ies'
    }
}).addValue({
    name: 'campus',
    table: 'vagas_ofertadas',
    tableField: 'campus',
    where: {
        relation: '=',
        type: 'string',
        field: 'campus'
    }
}).addValue({
    name: 'course',
    table: 'vagas_ofertadas',
    tableField: 'curso',
    where: {
        relation: '=',
        type: 'string',
        field: 'curso'
    }
}).addValue({
    name: 'shift',
    table: 'vagas_ofertadas',
    tableField: 'turno',
    where: {
        relation: '=',
        type: 'string',
        field: 'turno'
    }
}).addValue({
    name: 'name_ies',
    table: 'vagas_ofertadas',
    tableField: 'nome_ies',
    where: {
        relation: '=',
        type: 'string',
        field: 'nome_ies'
    }
}).addValue({
    name: 'year',
    table: 'vagas_ofertadas',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'min_year',
    table: 'vagas_ofertadas',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: 'vagas_ofertadas',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo'
    }
});

vacancyApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.field("SUM (CASE WHEN vagas_ofertadas.vagas_ampla_concorrencia_nao_pcd > 0 THEN vagas_ofertadas.vagas_ampla_concorrencia_nao_pcd ELSE 0 END) AS total_vagas_ampla_concorrencia_nao_pcd")
    .field("SUM (CASE WHEN vagas_ofertadas.vagas_ampla_concorrencia_pcd > 0 THEN vagas_ofertadas.vagas_ampla_concorrencia_pcd ELSE 0 END) AS total_vagas_ampla_concorrencia_pcd")
    .field("SUM (CASE WHEN vagas_ofertadas.vagas_cotas_renda_baixa_ppi_nao_pcd > 0 THEN vagas_ofertadas.vagas_cotas_renda_baixa_ppi_nao_pcd ELSE 0 END) AS total_vagas_cotas_renda_baixa_ppi_nao_pcd")
    .field("SUM (CASE WHEN vagas_ofertadas.vagas_cotas_renda_baixa_ppi_pcd > 0 THEN vagas_ofertadas.vagas_cotas_renda_baixa_ppi_pcd ELSE 0 END) AS total_vagas_cotas_renda_baixa_ppi_pcd")
    .field("SUM (CASE WHEN vagas_ofertadas.vagas_cotas_renda_baixa_outros_nao_pcd > 0 THEN vagas_ofertadas.vagas_cotas_renda_baixa_outros_nao_pcd ELSE 0 END) AS total_vagas_cotas_renda_baixa_outros_nao_pcd")
    .field("SUM (CASE WHEN vagas_ofertadas.vagas_cotas_renda_baixa_outros_pcd > 0 THEN vagas_ofertadas.vagas_cotas_renda_baixa_outros_pcd ELSE 0 END) AS total_vagas_cotas_renda_baixa_outros_pcd")
    .field("SUM (CASE WHEN vagas_ofertadas.vagas_cotas_renda_alta_ppi_nao_pcd > 0 THEN vagas_ofertadas.vagas_cotas_renda_alta_ppi_nao_pcd ELSE 0 END) AS total_vagas_cotas_renda_alta_ppi_nao_pcd")
    .field("SUM (CASE WHEN vagas_ofertadas.vagas_cotas_renda_alta_ppi_pcd > 0 THEN vagas_ofertadas.vagas_cotas_renda_alta_ppi_pcd ELSE 0 END) AS total_vagas_cotas_renda_alta_ppi_pcd")
    .field("SUM (CASE WHEN vagas_ofertadas.vagas_cotas_renda_alta_outros_nao_pcd > 0 THEN vagas_ofertadas.vagas_cotas_renda_alta_outros_nao_pcd ELSE 0 END) AS total_vagas_cotas_renda_alta_outros_nao_pcd")
    .field("SUM (CASE WHEN vagas_ofertadas.vagas_cotas_renda_alta_outros_pcd > 0 THEN vagas_ofertadas.vagas_cotas_renda_alta_outros_pcd ELSE 0 END) AS total_vagas_cotas_renda_alta_outros_pcd")
    .field("SUM (CASE WHEN vagas_ofertadas.total_pcd > 0 THEN vagas_ofertadas.total_pcd ELSE 0 END) AS total_total_pcd")
    .from("vagas_ofertadas")
    next();
}, query, response('evader'));

module.exports = vacancyApp;