/*
Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const debug = require('debug')('simcaq-api');
const libs = `${process.cwd()}/libs`;
const config = require(`${libs}/config`);
const log = require(`${libs}/log`)(module);
const app = require(`${libs}/app`);
const compatVersion = require(`${libs}/middlewares/checkVersion`);
const cluster = require('cluster');

// Check if Node version is compatible
if (!compatVersion()) {
    process.exit(1);
}

if(cluster.isMaster) {
    log.info(`Master ${process.pid} is running`);

    const numCPUs = (process.env.NODE_ENV != 'development') ? require('os').cpus().length : 1;
    log.info(`Master will create ${numCPUs} workers`);
    for(let i=0; i < numCPUs; ++i) {
        cluster.fork();
    }

    // Caso uma instâcia morra
    cluster.on('exit',  (worker, code, signal) => {
        log.info(`Worker ${worker.process.pid} died`);
        // Revive a instância
        cluster.fork();
    });
} else {
    // Set default port: first environment variable PORT, then configuration and last 3000
    app.set('port', process.env.PORT || config.port || 3000);
    process.env.NODE_ENV = process.env.NODE_ENV || 'development';
    // Set default ip: first environment variable IOP, then configuration and last '127.0.0.1'
    app.set('ip', process.env.IP || config.ip || '127.0.0.1');

    const server = app.listen(app.get('port'), () => {
        log.info(`Express server listening on port ${server.address().port}`);
    });

    log.info(`Worker ${cluster.worker.id} is running (${process.pid})`);

    // For testing
    module.exports = server;
}
