/*
Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

process.env.NODE_ENV = 'test';

const chai = require('chai');

const dirtyChai = require('dirty-chai');

chai.use(dirtyChai);

const chaiXml = require('chai-xml');

chai.use(chaiXml);

const chaiHttp = require('chai-http');

const assert = chai.assert;

const expect = chai.expect;

const should = chai.should(); // actually call the function

const libs = `${process.cwd()}/libs`;

const server = require(`${libs}/app`);

chai.use(chaiHttp);
describe('request classCount', () => {
    it('should return 400 with no filters', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/')
            .end((err, res) => {
                res.should.have.status(400);
                res.should.be.json;
                res.body.should.have.property('error');
                res.body.error.should.be.equal('Wrong/No filter specified');
                done();
            });
    });

    it('should list the rural locations', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/rural_location')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('id');
                res.body.result[0].should.have.property('name');
                done();
            });
    });

    it('should list the education level mod', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/education_level_mod')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('id');
                res.body.result[0].should.have.property('name');
                done();
            });
    });

    it('should list the education level short', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/education_level_short')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('id');
                res.body.result[0].should.have.property('name');
                done();
            });
    });

    it('should list the year range', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/year_range')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('start_year');
                res.body.result[0].should.have.property('end_year');
                done();
            });
    });

    it('should list the years', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/years')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('year');
                done();
            });
    });

    it('should list the administrative dependencies', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/adm_dependency')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('id');
                res.body.result[0].should.have.property('name');
                done();
            });
    });

    it('should list the administrative dependencies detailed', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/adm_dependency_detailed')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('id');
                res.body.result[0].should.have.property('name');
                done();
            });
    });

    it('should list the locations', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/location')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('id');
                res.body.result[0].should.have.property('name');
                done();
            });
    });

    it('should list the source', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/source')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('source');
                done();
            });
    });

    it('should list the simcaq class count', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/count')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('year');
                res.body.result[0].should.have.property('average');
                res.body.result[0].should.have.property('median');
                res.body.result[0].should.have.property('stddev');
                res.body.result[0].should.have.property('first_qt');
                res.body.result[0].should.have.property('third_qt');
                done();
            });
    });

    it('should list the simcaq class count with valid filter', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/count?filter=state:14')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('year');
                res.body.result[0].should.have.property('average');
                res.body.result[0].should.have.property('median');
                res.body.result[0].should.have.property('stddev');
                res.body.result[0].should.have.property('first_qt');
                res.body.result[0].should.have.property('third_qt');
                done();
            });
    });

    it('should list the simcaq class count with valid dim', (done) => {
        chai.request(server)
            .get('/api/v1/class_count/count?dims=region')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('year');
                res.body.result[0].should.have.property('average');
                res.body.result[0].should.have.property('median');
                res.body.result[0].should.have.property('stddev');
                res.body.result[0].should.have.property('first_qt');
                res.body.result[0].should.have.property('third_qt');
                res.body.result[0].should.have.property('region_name');
                done();
            });
    });

    it('should list class count with education level filter', (done) => {
        chai.request(server)
            .get('/api/v1/class_count?filter=education_level_mod:["1"]')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('year');
                res.body.result[0].should.have.property('average');
                res.body.result[0].should.have.property('median');
                res.body.result[0].should.have.property('stddev');
                res.body.result[0].should.have.property('first_qt');
                res.body.result[0].should.have.property('third_qt');
                done();
            });
    });

    it('should list class count with education level dims', (done) => {
        chai.request(server)
            .get('/api/v1/class_count?dims=education_level_mod')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('year');
                res.body.result[0].should.have.property('average');
                res.body.result[0].should.have.property('median');
                res.body.result[0].should.have.property('stddev');
                res.body.result[0].should.have.property('first_qt');
                res.body.result[0].should.have.property('third_qt');
                res.body.result[0].should.have.property('education_level_mod_id');
                res.body.result[0].should.have.property('education_level_mod_name');
                done();
            });
    });

    it('should list class count with valid dims', (done) => {
        chai.request(server)
            .get('/api/v1/class_count?dims=education_level_mod,location')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('year');
                res.body.result[0].should.have.property('average');
                res.body.result[0].should.have.property('median');
                res.body.result[0].should.have.property('stddev');
                res.body.result[0].should.have.property('first_qt');
                res.body.result[0].should.have.property('third_qt');
                res.body.result[0].should.have.property('education_level_mod_id');
                res.body.result[0].should.have.property('education_level_mod_name');
                res.body.result[0].should.have.property('location_id');
                res.body.result[0].should.have.property('location_name');
                done();
            });
    });

    it('should list class count with education level year limiter', (done) => {
        chai.request(server)
            .get('/api/v1/class_count?dims=education_level_mod&filter=min_year:2014,max_year:2014')
            .end((err, res) => {
                res.should.have.status(200);
                res.should.be.json;
                res.body.should.have.property('result');
                res.body.result.should.be.a('array');
                res.body.result[0].should.have.property('year');
                res.body.result[0].should.have.property('average');
                res.body.result[0].should.have.property('median');
                res.body.result[0].should.have.property('stddev');
                res.body.result[0].should.have.property('first_qt');
                res.body.result[0].should.have.property('third_qt');
                res.body.result[0].should.have.property('education_level_mod_id');
                res.body.result[0].should.have.property('education_level_mod_name');
                done();
            });
    });

});    
